/*
 * http://jqueryui.com/autocomplete/#combobox
 * http://www.learningjquery.com/2010/06/a-jquery-ui-combobox-under-the-hood
 * http://api.jqueryui.com/1.8/autocomplete/
 */
(function( $ ) {
    $.widget( "custom.combobox", {
        options: {
            title: 'Show all items'
        },
        _create: function() {
            this.wrapper = $( "<span>" )
                .addClass( "custom-combobox" )
                .insertAfter( this.element );

            this.element.hide();
            this._createAutocomplete();
            this._createShowAllButton();
        },

        _createAutocomplete: function() {
            var selected = this.element.children( ":selected" ),
                value = selected.val() ? selected.text() : "";

            this.input = $( "<input>" )
                .appendTo( this.wrapper )
                .val( value )
                .attr( "title", "" )
                .addClass( "custom-combobox-input ui-widget ui-widget-content ui-state-default ui-corner-left" )
                .on('click', function() {$(this).select();})
                .autocomplete({
                    delay: 0,
                    minLength: 0,
                    source: $.proxy( this, "_source" )
                });
            // tooltip no soportado en jQueryUI 1.8
            if (typeof this.input.tooltip == 'function')
                this.input.tooltip({
                    tooltipClass: "ui-state-highlight"
                });

            this.input.on({
                autocompleteselect: function( event, ui ) {
                    ui.item.option.selected = true;
                    $(this).trigger( "select", event, {
                        item: ui.item.option
                    });
                },

                autocompletechange: $.proxy(this, "_removeIfInvalid")
            });
        },

        _createShowAllButton: function() {
            var input = this.input,
                wasOpen = false;

            this.showallbutton = $( "<a>" )
                .attr( "tabIndex", -1 )
                .attr( "title", this.options.title )
                .appendTo( this.wrapper )
                .button({
                    icons: {
                        primary: "ui-icon-triangle-1-s"
                    },
                    text: false
                })
                .removeClass( "ui-corner-all" )
                .addClass( "custom-combobox-toggle ui-corner-right" )
                .mousedown(function() {
                    wasOpen = input.autocomplete( "widget" ).is( ":visible" );
                })
                .click(function() {
                    input.focus();

                    // Close if already visible
                    if ( wasOpen ) {
                        return;
                    }

                    // Pass empty string as value to search for, displaying all results
                    input.autocomplete( "search", "" );
                });
            // tooltip no soportado en jQueryUI 1.8
            if (typeof this.showallbutton.tooltip == 'function')
                this.showallbutton.tooltip();
        },

        _refresh: function() {
            this.showallbutton
                .attr('title', this.options.title);
        },

        _source: function( request, response ) {
            var matcher = new RegExp( $.ui.autocomplete.escapeRegex(request.term), "i" );
            response( this.element.children( "option" ).map(function() {
                var text = $( this ).text();
                if ( this.value && ( !request.term || matcher.test(text) ) )
                    return {
                        label: text,
                        value: text,
                        option: this
                    };
            }) );
        },

        _removeIfInvalid: function( event, ui ) {

            // Selected an item, nothing to do
            if ( ui.item ) {
                return;
            }

            // Search for a match (case-insensitive)
            var value = this.input.val(),
                valueLowerCase = value.toLowerCase(),
                valid = false;
            this.element.children( "option" ).each(function() {
                if ( $( this ).text().toLowerCase() === valueLowerCase ) {
                    this.selected = valid = true;
                    return false;
                }
            });

            // Found a match, nothing to do
            if ( valid ) {
                return;
            }

            // Remove invalid value
            this.input
                .val( "" )
                .attr( "title", value + " didn't match any item" );
            this.element.val( "" );
            // tooltip no soportado en jQueryUI 1.8
            if (typeof this.input.tooltip == 'function') {
                this.input.tooltip("open");
                // TODO: this._delay no existe en jQueryUI 1.8 pero tampoco
                // existe tooltip, así que no debería producir error.
                this._delay(function() {
                    this.input.tooltip( "close" ).attr( "title", "" );
                }, 2500 );
            }
            this.input.autocomplete( "instance" ).term = "";
        },

        _destroy: function() {
            this.wrapper.remove();
            this.element.show();
        },

        _setOptions: function() {
            // _super and _superApply handle keeping the right this-context
            this._superApply( arguments );
            this._refresh();
        },
    });
})( jQuery );
